@@ -9,7 +9,7 @@ def get_instances_of(type, *bases):
             if isinstance(value, type):
                 yield value
         for value in get_instances_of(type, *base.__bases__):
-            yield value            
+            yield value
 
 class Validator(object):
     """Wrapper for validators.
@@ -47,7 +47,7 @@ class Action(object):
     def __repr__(self):
         return '<%s name="%s" submitted="%s">' % (
             type(self).__name__, self.name or "", str(bool(self.submitted)))
-    
+
 class metaclass(type):
     def __init__(kls, name, bases, dict):
         kls.validators = tuple(get_instances_of(Validator, kls))
@@ -58,12 +58,13 @@ class Form(object):
     WebOb-like request object as ``request``."""
 
     __metaclass__ = metaclass
-    
+
     fields = {}
     status = None
     prefix = None
-    
-    def __init__(self, data=None, context=None, request=None, prefix=None):
+    action = None
+
+    def __init__(self, data=None, context=None, request=None, params=None, prefix=None):
         self.context = context
         self.request = request
 
@@ -79,12 +80,18 @@ class Form(object):
 
         if prefix is None:
             prefix = self.prefix
-            
+
+        if request is not None:
+            if params is not None:
+                raise ValueError(
+                    "Can't provide both ``params`` and ``request``.")
+            params = request.params
+
         # find action parameters
         action_params = {}
-        if prefix is not None:
+        if prefix is not None and params is not None:
             re_prefix = re.compile(r'^%s[._-](?P<name>.*)' % prefix)
-            for key, value in request.params.items():
+            for key, value in params.items():
                 if key == prefix:
                     action_params[None] = value
                 else:
@@ -96,18 +103,20 @@ class Form(object):
         actions = self.actions = []
         for action in type(self).actions:
             name = action.name
-            actions.append(
-                Action(action.__call__, name, name in action_params))
-                
+            action = Action(action.__call__, name, name in action_params)
+            actions.append(action)
+            if action:
+                self.action = action
+
         # conditionally apply request parameters if:
         # 1. no prefix has been set
         # 2. there is a submitted action
         # 3. there are no defined actions, but a default action was submitted
-        if request is not None and (
+        if params is not None and (
             prefix is None or \
             filter(None, actions) or \
             len(actions) == 0 and action_params.get(None) is not None):
-            params = request.params.items()
+            params = params.items()
         else:
             params = ()
 
@@ -123,11 +132,10 @@ class Form(object):
     def __call__(self):
         """Calls the first submitted action and returns the value."""
 
-        for action in self.actions:
-            if action:
-                status = self.status = action(self, self.data)
-                return status
-            
+        if self.action is not None:
+            self.status = self.action(self, self.data)
+        return self.status
+
     def validate(self):
         """Validates the request against the form fields. Returns
         ``True`` if all fields validate, else ``False``."""
