@@ -1,5 +1,4 @@
-from collections import defaultdict
-
+from repoze.formapi.py24 import defaultdict
 from repoze.formapi.error import Errors
 
 MISSING = object()
@@ -165,7 +164,7 @@ def marshall(params, fields):
             e = errors
             for p in path:
                 e = e[p]
-            e += error.message
+            e += str(error)
         except KeyError:
             # parameter does not match this form field definition; we
             # can safely disregard it.
